#include <puppy.h>
#include <pupmem.h>
#include <ascii.h>

/* Main FidoNet machine; FidoNet Standards Committee (FSC001-7); the 
printf()'s display FSC001 FSM states at the appropriate places. */

rcv_mail() {
struct _pkthdr pkthdr;			/* received packet header */
struct _node d;				/* node address of sender (from packet) */
struct _tm tm;				/* topic map for doing pickup */

int i;

	set_abort(0);				/* trap carrier loss here, */
	if (was_abort()) {
		cd_flag= 0;			/* real CD, */
		if (cd()) discon();		/* hang up if still connected, */
		return;
	}

/* R0 (WaitCxD), and R1 (WaitBaud), are done before we even
get here, in fidonet(), by answer(). */

	limit= 0;				/* no limit */
	cd_flag= 0;				/* watch Carrier Detect */
	clr_clk();				/* reset the clock, */
	flush(0);				/* clear the modem */

	printf("Pup says: \"R1: Incoming Call\"\r\n");

	for (i= 5; i--;) {			/* send CRs so the remote */
		modout(CR);			/* knows we completed */
		delay(10);			/* R0 and R1 (just in case ...) */
	}
	mprintf("Pup says: \"Yo dude! FidoNet only! Go away! Call again soon!\"\r\n");

/* R2, WaitTSYNC */

	wait_tsync();				/* wait for TSYNC */

/* R3, RecMail; states R3 - R6 in recv_pkt() */

	if (! recv_pkt(&pkthdr)) frc_abort();	 /* get packet, files */

/* R7, AllowPickup */

	d.number= pkthdr.orig_number;		/* make a real node */
	d.net= pkthdr.orig_net;			/* structure from the */
	d.zone= pup.id.zone;			/* packet horror */

/* Try to locate the topic map for the node that dropped off mail, to see
if there is any mail for pickup. If there isnt any, then findtm() will
return -1, so we fill in a fake topic map entry to force send_pkt() to send
out an empty packet to satisfy the pickup-er. Since the index will be -1
we wont update anything. */

	i= findtm(&tm,&d);			/* locate this topic map, */
	if (i == -1) tm.topic= 0;		/* no mail to this node */
	printf("Pup says: \"R7: Pickup\"\r\n");	/* i is -1 if no mail for pickup */
	send_pkt(&tm);				/* send it (maybe empty packet) */
	if (i != -1) puttm(&tm,i);		/* update the record if OK, */
	frc_abort();				/* error trap exit */
}

/* FSC001 transmit mail. As above, FSC001 states are displayed at various
points along the way. */

send_mail(recd)
int recd;
{
struct _tm tm;			/* topic map for this call */
struct _pkthdr pkthdr;		/* packet header from (possibly) picked up mail */
int n,i;

	gettm(&tm,recd);			/* get node info, */

	set_abort(0);
	if (was_abort()) {			/* after completion */
		++tm.tries;			/* log the attempt, */
		if (! cd_flag) {		/* if we connected, */
			tm.connect_time += (minutes * 60) + seconds;
			++tm.connects;		/* log that, */
		}
		puttm(&tm,recd);		/* update it */
		cd_flag= 0;
		if (cd()) discon();		/* disconnect if still online */
		return;
	}

	cd_flag= 1;				/* not connected yet */
	clr_clk();				/* reset the clock, */
	limit= 2;				/* failsafe dial killer, */

/* S0 SendInit, S1 WaitCxD combined */

/* [
	get_ndat(&tm.node);			/* get physical info */
	printf("\r\nPup says: \"S0/S1: Calling FidoNet %s, tel. %s\r\n",str_node(&ndat.node),ndat.phone);
	setbaud(ndat.rate);			/* set dialing rate, */
	i= dial(ndat.phone);			/* dial the number, */
 ] */

/* ALPHA */
	printf("\r\nPup says: \"S0/S1: Calling FidoNet %s, tel. %s\r\n",str_node(&tm.node),tm.phone);
	setbaud(tm.baud);			/* set dialing rate, */
	i= dial(tm.phone);			/* dial the number, */
/* ALPHA */

	printf("Pup says: \"%s\"\r\n",lastconnect());	/* log the attempt, */
	switch (i) {
		case 0: frc_abort(); break;	/* no connection */
		case 1: break;			/* connected now */
		default: 			/* -1 == error */
			tm.tries= pup.tries; 
			frc_abort(); break;
	}
	delay(400);				/* let modems settle */

/* End S0/S1 */

/* Proprietary smartass cleanup and overhead */

	clr_clk();				/* reset clock, */
	limit= 1;				/* failsafe on WhackCR */
	cd_flag= 0;				/* now watch carrier */
	flush(0);				/* flush buffers */

/* S2 WhackCRs */

	printf("Pup says: \"S2: Whack CRs\"\r\n");
	for (n= 20; --n;) {			/* limited tries for baud test */
		modout(CR);
		delay(20);			/* get one back, */
		modout(' ');			/* send CR space until we */
		delay(20);
		modout(ETX);			/* ^C to kill welcomes */
		while (1) {
			i= modin(100);		/* get a character */
			if (i == CR) break;	/* either CR */
			if (i == -1) break;	/* or time out */
		}
		if (i == CR) break;		/* in sync now */
	}
	if (n == 0) {
		printf("Pup says: \"Whack CR failed\"\r\n");
		frc_abort();
	}
	delay(100);				/* dont shit on XMODEM */

/* End S2 */

/* States S3 - S6 in send_pkt() */

/* Need magic translate ndat.node to topic */

	if (! send_pkt(&tm)) {			/* send, mark complete */
		printf("Pup says: \"Error sending packet\"\r\n");
		frc_abort();
	}

/* S8 TryPickup */

	printf("Pup says: \"S8: Trying Mail Pickup\"\r\n");
	tm.connect_time += (minutes * 60) + seconds;
	clr_clk();				/* connect time so far */
	limit= 2;				/* time to sync, etc */
	wait_tsync();				/* wait for TSYNC */
	recv_pkt(&pkthdr);			/* get packets and files */
	frc_abort();				/* do termination code */
}

/* States R3 - R5. We are connected to an incoming call; receive the mail 
packet and any attached files. */

recv_pkt(pkthdr)
struct _pkthdr *pkthdr;
{
int i;
char *cp,buff[SS];
struct _node d;

	printf("Pup says: \"R3: Receiving Mail Packet\"\r\n");

	totl_files= 0;
	limit= 0;				/* no time limit */
	crcmode= 1;				/* CRC, */
	filemode= FIDONET;			/* XMODEM, */
	mailfrom(pkthdr);			/* start the unpacketer, */
	receive("NUL",0,text,textsize);		/* receive" a packet */

/* R4, XRecEnd */

	if (! totl_files) {
		printf("Pup says: \"R4: Never got no packets! Nope!\"\r\n");
		return(0);
	}
	printf("Pup says: \"R4: Received packet\"\r\n");
	delay(100);				/* delay for EOTs to settle */
	flush(0);				/* flush rubbish */

/* R5, RecFiles */

	totl_files= 0;
	printf("Pup says: \"R5: Waiting for any attached file(s)\"\r\n");
	crcmode= 1;
	filemode= TELINK;
	*text= NUL;				/* clear list of received files */
	receive("",text,textsize);		/* attached files, */
	printf("Pup says: \"Received %d files\"\r\n",totl_files);

/* R6, ChkFiles; success checked by caller */

	if (totl_files > 0) {
		i= append("FILES.PUP");		/* add to the list, */
		if (i == -1) {
			printf("Pup says: \"Can't append filenames to FILES.PUP!\"\r\n");
			return(0);
		}

		d.number= pkthdr-> orig_number;	/* make a real node */
		d.net= pkthdr-> orig_net;	/* structure from the */
		d.zone= pup.id.zone;		/* packet horror */

		cp= text;			/* put each in the list */
		while (*cp) {
			cpyarg(buff,cp);
			write(i,buff,strlen(buff));
			sprintf(buff," from %s\r\n",str_node(&d));
			if (write(i,buff,strlen(buff)) != strlen(buff)) {
				close(i);
				printf("Pup says: \"Disk Full adding names to FILES.PUP!\"\r\n");
				return(0);
			}
			cp= next_arg(cp);
		}
		close(i);
	}
	return(1);
}

/* Wait for a TSYNC, return if found. Limitchk() will get it if we timeout. */

wait_tsync() {
int i;

/* R2, WaitTSYNC */

	printf("Pup says: \"R2: Waiting for TSYNC\"\r\n");
	limit= 2;
	clr_clk();					/* 60 sec limit */

	while (1) {
		i= modin(100);
		switch (i) {
			case TSYNC: return;		/* got it */

/* Proprietary: send CR if CR is returned; this covers the case
where the sender is still in state S2, and we want it to be in S3. */

			case CR: 
				printf("Pup says: \"Sender still in S2\"\r\n");
				modout(CR); 		/* missed whack CR? */
				break;
		}
		flush(0);
	}
}

/* States S3 - S6: send mail to the current node; return packets 
and files set properly. */

send_pkt(tm)
struct _tm *tm;
{

/* S3 WaitClear */

/* NOTE: This does only one TSYNC, as per the spec, but not per the
reccommendation. */

	printf("Pup says: \"S3: Waiting for clear line\"\r\n");
	limit= 1;				/* wait for quiet line */
	clr_clk();
	flush(50); 				/* wait for 1/2 sec quiet line, */
	limit= 0;				/* got it, now no time limit */
	modout(TSYNC);				/* send our sync, */

/* S4 SendMail */

	printf("Pup says: \"S4: Sending Mail Packet\"\r\n");

	limit= 0;				/* no limit now */
	totl_files= 0;
	crcmode= 1;
	filemode= FIDONET;			/* XMODEM/byte diverter */
	mailfor(tm);				/* start the packeter, */
	transmit("NUL",0,text,textsize);	/* send "packet file" */

/* S5, CheckMail */

	if (! totl_files) {
		printf("Pup says: \"S5: Mail packet not sent!\"\r\n");
		return(0);			/* terminate if nothing sent */
	}

/* This step marks this node as updated, and takes the place of all
that awful SEEN-BY stuff. */

	gtod(&tm-> time);			/* mark as updated */

	printf("Pup says: \"S5: Mail packet sent\r\n");
	delay(100);				/* delay to let rcvr finish */

/* S6 SendFiles */

/* transmit() will handle 0 files, ie. do the "no more files"
protocol to tell the receiver that there are no attached files.. */

	printf("Pup says: \"S6: Fake attached files\"\r\n");
	totl_files= 0;
	filemode= TELINK;			/* now send any files, */
	transmit("",0,text,textsize);		/* relative to default dir */
	printf("Pup says: \"Of course we sent %d files\"\r\n",totl_files);

/* S7, CheckFile */

	printf("Pup says: \"S7: File attach complete\"\r\n");

/* End S6 */
}
