#include <puppy.h>
#include <pupmem.h>
#include <ascii.h>

/*
	Puppy's Packeter.

What it really does is generate the packet on the fly as XMODEM requests 
bytes to "send the mail packet". A state machine keeps track of where it 
is, and generates each portion of the packet as it goes, for the 
convenience of the programmer. (Namely, me.)

No packet files, no disk overhead, no filename management, no seperate 
packeting step, instant pickup, simple routing, no strain on primitive
file systems ... more online processing time, more XMODEM timeouts, 
file transfer effeciency loss, especially in pipelined protocols and
modems. 


NOTE: This uses the global buffer "text" for work space between iterations.



	mailfor(node,topic);
	struct _node *node;
	WORD topic;

	This clears the state machine for "packet file" generation
	via subsequent calls to netbyte().


	get_pkt()

	This returns bytes from the "packet file", returning 8 bit
	bytes until EOF is reached, when -1 is returned. This actually
	does the byte generation via the state machine.

*/

/* These dont HAVE TO be static, but no one else references them. */

static struct _pkthdr pkthdr;		/* we need this */
static struct _node node;		/* packet destination node address */

static char state;			/* state machine */
static int msgnbr;			/* "current message number" for packeting/sending messages */
static WORD topic;			/* current topic/node we're working on */
static char *ptr;			/* ptr we return bytes from */
static unsigned cnt;			/* how many ... */

/* Set the desired topic(s), destination node number and reset the
state machine. Return true if this node has any messages. */

mailfor(tm)
struct _tm *tm;
{
int n;
	topic= tm-> topic;			/* remember these */
	cpy_node(&node,&tm-> node);		/* and this */
	state= 0;				/* reset this */

	for (n= 0; n= nxtmsg(n,topic); ) {	/* look for new msgs */
		if (after(&getmsg(n)-> time,&tm-> time)) return(1);
	}
	return(0);				/* no new messages found */
}

/* This is the packet byte generator. */

get_pkt() {
int i;
char *cp;
long l;						/* dont trust Lattice 2.1 */
struct _msg *m;					/* working msg pointer */
char buff[SS];
WORD time,date;

	if (cnt) return(get_byte());	/* return bytes ... */

	switch (state) {			/* ... then generate some more */

/* Packet header: Fill in the structure with our stuff, and return
it a byte at a time. */

		case 0:
			cp= (char *) &pkthdr;
			for (i= sizeof(struct _pkthdr); --i;) *cp++= NUL;

			pkthdr.orig_number= pup.id.number;
			pkthdr.orig_net= pup.id.net;
			pkthdr.orig_zone= pup.id.zone;

			pkthdr.dest_number= node.number;
			pkthdr.dest_net= node.net;
			pkthdr.dest_zone= node.zone;

			date= gdate(); time= gtime();
			pkthdr.year= (date >> 9) & 0x3f;
			pkthdr.month= (date >> 5) & 0x0f;
			pkthdr.day= date & 0x1f;
			pkthdr.hour= time >> 11;
			pkthdr.minute= (time >> 5) & 0x3f;
			pkthdr.ver= PKTVER;

			state= 1;			/* set for next state */
			msgnbr= 0;			/* start with oldest msg */

			ptr= (char *) &pkthdr;	/* bytes we've "generated" */
			cnt= sizeof(pkthdr);
			break;

/* Generate the message-header data. The machine switches between this state
and the message-body state until there are no more messages. "Unsent" messages
are those that do not have the corresponding topic_map bit(s) set. The 
topic_map flags which topics this message has been transmitted sucessfully to. */

		case 1:
			msgnbr= nxtmsg(msg,topic);	/* find the next one, */
			if (msgnbr < newest_msg()) {	/* if no more messages, */
				state= 3;		/* next is EOF state */
				clrb();
				putw(0);		/* generate msg type 0 */
				break;
			}
			clrb();				/* clear the buffer, */
			putw(2);			/* message type = 2 */
			putw(pup.id.number);		/* msg source node */
			putw(node.number);		/* msg dest node */
			putw(pup.id.net);		/* msg source net */
			putw(node.net);			/* msg dest net */
			putw(0);			/* Fido attributes */
			putw(0);			/* Fido cost */
			m= getmsg(msgnbr);		/* get ptr to the msg */
			strcpy(buff,str_date(m-> date));
			strcat(buff," ");
			strcat(buff,str_time(m-> time));
			putst(buff);
			putst(m-> to);
			putst(m-> from);
			putst(m-> subj);
			state= 2;			/* next state is msg-body */
			break;

/* Generate message-body bytes. We stay in this state until the message
is empty, then we switch back to state 1. */

		case 2:
			loadmsg(msgnbr);		/* get the msg body */
			clrb();				/* count the bytes */
			while (*ptr++ != SUB) ++cnt;	/* we just loaded */
			ptr= text;			/* reset the pointer, */
			putw(0);			/* mark the end, */
			state= 1;			/* then do next msg */
			break;

/* No more messages; mark the end of the packet. */

		case 3:
			clrb();
			putw(0);
			state= 4;
			break;

/* End of file for XMODEM. */

		case 4:
			state= 0;
			return(-1);			/* EOF marker */
	}
	return(get_byte());				/* always return one */
}

/* Clear the local buffer. */

clrb() {

	ptr= text;
	cnt= 0;
}

/* Put an integer into the stream, Little Endian. */

static putw(i)
WORD i;
{
	putb(i);			/* low byte, */
	putb(i >> 8);			/* high byte, */
}

/* Put a string into the stream, with a null tail. (ugh) */

static putst(s)
BYTE *s;
{
	while (*ptr++= *s++) ++cnt;
}

/* Write a byte to the stream. */

static putb(b)
BYTE b;
{
	*ptr++= b;
	++cnt;
}

/* Return a data byte from the previously set state. */

static get_byte() {

	--cnt;				/* count it, */
	return(*ptr++);			/* return it */
}

/* Return the next message number in either direction or zero if
no more. If zero is passed, start with the oldest message. */

nxtmsg(n,topic)
int n;
WORD topic;
{
	while (1) {
		if (n == 0) n= oldest_msg();		/* start the process */
		else if (--n < newest_msg()) return(0);	/* or check for no more */
		if (ismsg(n,topic)) return(n);		/* found one */
	}
}
