#include <puppy.h>
#include <pupmem.h>
#include <ascii.h>

/* 
Topic map "data base" functions. 

opentm()	Opens the topic map. Must be called before anything else.
		Returns the number of entries in the topic map, and clears
		out the variable info.

closetm()	Closes the topic map file, saves any changes.

gettm(tm,n)	Get topic map entry #N, loads it to the topic map 
struct _tm *tm;	pointed to by tm. No error checking is done.

puttm(tm,n)	Updates the topic map file record #N with the topic 
struct _tm *tm;	map structure passed. No error checking is done.

findtm(tm,d)	Locate FidoNet node d in the topic map, returns the 
struct _tm *tm;	topic map structure filled in and the index of it in
struct _node *d; the topic map file. Returns -1 if not found.

*/

#define TMAX 20			/* max. we will allow */
static struct _tm tmap[TMAX];	/* the memory-resident topic map */

/* Open the topic map file. Return the number of topic map entries. */

opentm() {
int f,n;

	f= open("puppy.tm",2);
	n= 0;
	if (f != -1) {
		n= read(f,&tmap,sizeof(tmap));
		n /= sizeof(struct _tm);		/* calc number of tms */
		close(f);

		for (f= 0; f < n; f++) {		/* clear out the */
			tmap[f].tries= 0;		/* stats */
			tmap[f].connects= 0;
			tmap[f].connect_time= 0;
		}

	} else printf("THE TOPIC MAP IS MISSING!\r\n");
	return(n);
}

/* Close the caller file. */

closetm() {
int f;

	f= open("puppy.tm",2);
	if (f != -1) {
		write(f,&tmap,sizeof(tmap));
		close(f);
	} else printf("ERROR WRITING THE TOPIC MAP!\r\n");
}

/* Get caller #n in the list. */

gettm(tm,n)
struct _tm *tm;
int n;
{

	if (n < TMAX) movemem(tm,&tmap[n],sizeof(struct _tm));
}

/* Get caller #n in the list. */

puttm(tm,n)
struct _tm *tm;
int n;
{
	if (n < TMAX) movemem(&tmap[n],tm,sizeof(struct _tm));
}


/* Locate node D in the topic map, load that structure and return its
index, or return -1 if not found. */

findtm(tm,d)
struct _tm *tm;
struct _node *d;
{
int n;

	for (n= 0; n < TMAX; n++) {
		if (same_node(d,&tmap[n].node)) {
			gettm(tm,n);
			return(n);
		}
	}
	return(-1);
}

/* Move some memory. */

movemem(t,f,n)
char *t,*f;	/* To and From pointers, */
unsigned n;	/* byte count */
{
	while (n--) *t++= *f++;
}
